@@ -3,7 +3,7 @@
 pacman -U --noconfirm dist/*.pkg.*
 
 #run the CI tests for Qt5
-pacman -S --noconfirm qt5-base
+pacman -S --noconfirm qt5-base gcovr
 meson qt5-thread-test contrib/ci/qt5-thread-test
 ninja -C qt5-thread-test test
 
@@ -18,7 +18,12 @@ export G_TEST_SRCDIR=/usr/share/installed-tests/fwupd G_TEST_BUILDDIR=/usr/share
 sleep 5
 
 fwupdtool enable-test-devices
-/usr/lib/fwupd/fwupd --verbose &
+# tag test device for emulation before starting daemon
+fwupdtool emulation-tag 08d460be0f1f9f128413f816022a6439e0078018
+G_DEBUG=fatal-criticals /usr/lib/fwupd/fwupd --verbose &
 sleep 10
 /usr/share/installed-tests/fwupd/fwupdmgr.sh
 /usr/share/installed-tests/fwupd/fwupdtool.sh
+
+# generate coverage report
+./contrib/ci/coverage.sh
