@@ -1,9 +1,17 @@
 #!/bin/bash -e
 
+# Set up fatal-criticals systemd override
+SYSTEMD_OVERRIDE="/etc/systemd/system/fwupd.service.d"
+mkdir -p ${SYSTEMD_OVERRIDE}
+cp contrib/fwupd-systemd-fatal-criticals.conf ${SYSTEMD_OVERRIDE}/override.conf
+
 # install
 apt update
 apt install -y gcovr ./dist/*.deb
 
+fwupdtool enable-test-devices
+fwupdtool emulation-tag 08d460be0f1f9f128413f816022a6439e0078018
+
 # run tests
 ./contrib/ci/get_test_firmware.sh
 cp fwupd-test-firmware/installed-tests/* /usr/share/installed-tests/fwupd/ -LRv
@@ -11,15 +19,7 @@ service dbus restart
 gnome-desktop-testing-runner fwupd
 
 # generate coverage report
-if [ "$CI" = "true" ]; then
-	gcovr -x \
-		--filter build/libfwupd \
-		--filter build/libfwupdplugin \
-		--filter build/plugins \
-		--filter build/src \
-		-o coverage.xml
-	sed "s,build/,,g" coverage.xml -i
-fi
+./contrib/ci/coverage.sh
 
 # cleanup
 apt purge -y fwupd fwupd-doc libfwupd3 libfwupd-dev
