@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/sh -e
 
 exec 2>&1
 
@@ -6,14 +6,13 @@ run_test()
 {
         if [ -f @installedtestsbindir@/$1 ]; then
                 @installedtestsbindir@/$1
-                rc=$?; if [ $rc != 0 ]; then exit $rc; fi
         fi
 }
 
 run_device_tests()
 {
 	if [ -n "$CI_NETWORK" ] && [ -d @devicetestdir@ ]; then
-		for f in `grep --files-with-matches -r emulation-url @devicetestdir@`; do
+		for f in `grep --files-with-matches -r emulation- @devicetestdir@`; do
 		        echo "Emulating for $f"
 		        fwupdmgr device-emulate \
 				--download-retries=5 \
@@ -22,7 +21,6 @@ run_device_tests()
 				--no-metadata-check \
 				--verbose \
 				"$f"
-		        rc=$?; if [ $rc != 0 ]; then exit $rc; fi
 		done
 		fwupdmgr quit
 	fi
@@ -36,7 +34,6 @@ run_umockdev_test()
 		TESTS=`${INSPECTOR} ${ARG}`
 		for test in ${TESTS}; do
 			${ARG} ${test} --verbose
-			rc=$?; if [ $rc != 0 ]; then exit $rc; fi
 		done
 	fi
 }
@@ -44,6 +41,7 @@ run_umockdev_test()
 run_test acpi-dmar-self-test
 run_test acpi-facp-self-test
 run_test acpi-phat-self-test
+run_test acpi-ivrs-self-test
 run_test ata-self-test
 run_test fwupdplugin-self-test
 run_test nitrokey-self-test
@@ -60,7 +58,6 @@ run_test mtd-self-test
 run_test vli-self-test
 run_device_tests
 run_umockdev_test fwupd_test.py
-run_umockdev_test amd_pmc_test.py
 run_umockdev_test pci_psp_test.py
 
 # success!
