@@ -222,7 +222,7 @@ fu_cfi_device_wait_for_status_cb(FuDevice *device, gpointer user_data, GError **
 			    FWUPD_ERROR_INTERNAL,
 			    "wanted 0x%x, got 0x%x",
 			    helper->value,
-			    buf[0x1] & helper->mask);
+			    (guint)(buf[0x1] & helper->mask));
 		return FALSE;
 	}
 
@@ -250,12 +250,17 @@ fu_cfi_device_wait_for_status(FuCfiDevice *self,
 static gboolean
 fu_cfi_device_read_jedec(FuCfiDevice *self, GError **error)
 {
+	FuCfiDeviceClass *klass = FU_CFI_DEVICE_GET_CLASS(self);
 	guint8 buf_res[] = {0x9F};
 	guint8 buf_req[3] = {0x0};
 	g_autoptr(FuDeviceLocker) cslocker = NULL;
 	g_autoptr(FuProgress) progress = fu_progress_new(G_STRLOC);
 	g_autoptr(GString) flash_id = g_string_new(NULL);
 
+	/* has to be handled by the baseclass */
+	if (klass->read_jedec != NULL)
+		return klass->read_jedec(self, error);
+
 	/* enable chip */
 	cslocker = fu_cfi_device_chip_select_locker_new(self, error);
 	if (cslocker == NULL)
@@ -274,14 +279,8 @@ fu_cfi_device_read_jedec(FuCfiDevice *self, GError **error)
 	}
 	if ((buf_req[0] == 0x0 && buf_req[1] == 0x0 && buf_req[2] == 0x0) ||
 	    (buf_req[0] == 0xFF && buf_req[1] == 0xFF && buf_req[2] == 0xFF)) {
-		g_set_error(error,
-			    FWUPD_ERROR,
-			    FWUPD_ERROR_NOT_SUPPORTED,
-			    "device not detected, flash ID 0x%02X%02X%02X",
-			    buf_req[0],
-			    buf_req[1],
-			    buf_req[2]);
-		return FALSE;
+		fu_cfi_device_set_flash_id(self, NULL);
+		return TRUE;
 	}
 	g_string_append_printf(flash_id, "%02X", buf_req[0]);
 	g_string_append_printf(flash_id, "%02X", buf_req[1]);
@@ -304,6 +303,8 @@ fu_cfi_device_setup(FuDevice *device, GError **error)
 		if (!fu_cfi_device_read_jedec(self, error))
 			return FALSE;
 	}
+	if (priv->flash_id == NULL)
+		return TRUE;
 
 	/* sanity check */
 	if (priv->flash_id != NULL)
@@ -331,6 +332,7 @@ fu_cfi_device_setup(FuDevice *device, GError **error)
 	}
 
 	/* success */
+	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UPDATABLE);
 	return TRUE;
 }
 
@@ -914,7 +916,10 @@ fu_cfi_device_write_firmware(FuDevice *device,
 	fu_progress_step_done(progress);
 
 	/* write each block */
-	pages = fu_chunk_array_new_from_bytes(fw, 0x0, fu_cfi_device_get_page_size(self));
+	pages = fu_chunk_array_new_from_bytes(fw,
+					      FU_CHUNK_ADDR_OFFSET_NONE,
+					      FU_CHUNK_PAGESZ_NONE,
+					      fu_cfi_device_get_page_size(self));
 	if (!fu_cfi_device_write_pages(self, pages, fu_progress_get_child(progress), error)) {
 		g_prefix_error(error, "failed to write pages: ");
 		return FALSE;
@@ -966,7 +971,6 @@ fu_cfi_device_init(FuCfiDevice *self)
 	priv->cmds[FU_CFI_DEVICE_CMD_CHIP_ERASE] = 0x60;
 	priv->cmds[FU_CFI_DEVICE_CMD_READ_ID] = 0x9f;
 	fu_device_add_protocol(FU_DEVICE(self), "org.jedec.cfi");
-	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UPDATABLE);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UNSIGNED_PAYLOAD);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_CAN_VERIFY_IMAGE);
 	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_USE_PARENT_FOR_OPEN);
