@@ -293,28 +293,28 @@ FuDevice *
 fu_device_new(FuContext *ctx);
 
 /* helpful casting macros */
-#define fu_device_has_flag(d, v)	   fwupd_device_has_flag(FWUPD_DEVICE(d), v)
-#define fu_device_has_flag(d, v)	   fwupd_device_has_flag(FWUPD_DEVICE(d), v)
-#define fu_device_has_request_flag(d, v)   fwupd_device_has_request_flag(FWUPD_DEVICE(d), v)
-#define fu_device_add_request_flag(d, v)   fwupd_device_add_request_flag(FWUPD_DEVICE(d), v)
-#define fu_device_has_instance_id(d, v)	   fwupd_device_has_instance_id(FWUPD_DEVICE(d), v)
-#define fu_device_has_vendor_id(d, v)	   fwupd_device_has_vendor_id(FWUPD_DEVICE(d), v)
-#define fu_device_has_protocol(d, v)	   fwupd_device_has_protocol(FWUPD_DEVICE(d), v)
-#define fu_device_has_checksum(d, v)	   fwupd_device_has_checksum(FWUPD_DEVICE(d), v)
-#define fu_device_add_checksum(d, v)	   fwupd_device_add_checksum(FWUPD_DEVICE(d), v)
-#define fu_device_add_release(d, v)	   fwupd_device_add_release(FWUPD_DEVICE(d), v)
-#define fu_device_add_icon(d, v)	   fwupd_device_add_icon(FWUPD_DEVICE(d), v)
-#define fu_device_has_icon(d, v)	   fwupd_device_has_icon(FWUPD_DEVICE(d), v)
-#define fu_device_add_issue(d, v)	   fwupd_device_add_issue(FWUPD_DEVICE(d), v)
-#define fu_device_set_description(d, v)	   fwupd_device_set_description(FWUPD_DEVICE(d), v)
-#define fu_device_set_flags(d, v)	   fwupd_device_set_flags(FWUPD_DEVICE(d), v)
-#define fu_device_set_plugin(d, v)	   fwupd_device_set_plugin(FWUPD_DEVICE(d), v)
-#define fu_device_set_serial(d, v)	   fwupd_device_set_serial(FWUPD_DEVICE(d), v)
-#define fu_device_set_summary(d, v)	   fwupd_device_set_summary(FWUPD_DEVICE(d), v)
-#define fu_device_set_branch(d, v)	   fwupd_device_set_branch(FWUPD_DEVICE(d), v)
-#define fu_device_set_update_error(d, v)   fwupd_device_set_update_error(FWUPD_DEVICE(d), v)
-#define fu_device_add_vendor_id(d, v)	   fwupd_device_add_vendor_id(FWUPD_DEVICE(d), v)
-#define fu_device_add_protocol(d, v)	   fwupd_device_add_protocol(FWUPD_DEVICE(d), v)
+#define fu_device_has_flag(d, v)	 fwupd_device_has_flag(FWUPD_DEVICE(d), v)
+#define fu_device_has_flag(d, v)	 fwupd_device_has_flag(FWUPD_DEVICE(d), v)
+#define fu_device_has_request_flag(d, v) fwupd_device_has_request_flag(FWUPD_DEVICE(d), v)
+#define fu_device_add_request_flag(d, v) fwupd_device_add_request_flag(FWUPD_DEVICE(d), v)
+#define fu_device_has_instance_id(d, v)	 fwupd_device_has_instance_id(FWUPD_DEVICE(d), v)
+#define fu_device_has_vendor_id(d, v)	 fwupd_device_has_vendor_id(FWUPD_DEVICE(d), v)
+#define fu_device_has_protocol(d, v)	 fwupd_device_has_protocol(FWUPD_DEVICE(d), v)
+#define fu_device_has_checksum(d, v)	 fwupd_device_has_checksum(FWUPD_DEVICE(d), v)
+#define fu_device_add_checksum(d, v)	 fwupd_device_add_checksum(FWUPD_DEVICE(d), v)
+#define fu_device_add_release(d, v)	 fwupd_device_add_release(FWUPD_DEVICE(d), v)
+#define fu_device_add_icon(d, v)	 fwupd_device_add_icon(FWUPD_DEVICE(d), v)
+#define fu_device_has_icon(d, v)	 fwupd_device_has_icon(FWUPD_DEVICE(d), v)
+#define fu_device_add_issue(d, v)	 fwupd_device_add_issue(FWUPD_DEVICE(d), v)
+#define fu_device_set_description(d, v)	 fwupd_device_set_description(FWUPD_DEVICE(d), v)
+#define fu_device_set_flags(d, v)	 fwupd_device_set_flags(FWUPD_DEVICE(d), v)
+#define fu_device_set_plugin(d, v)	 fwupd_device_set_plugin(FWUPD_DEVICE(d), v)
+#define fu_device_set_serial(d, v)	 fwupd_device_set_serial(FWUPD_DEVICE(d), v)
+#define fu_device_set_summary(d, v)	 fwupd_device_set_summary(FWUPD_DEVICE(d), v)
+#define fu_device_set_branch(d, v)	 fwupd_device_set_branch(FWUPD_DEVICE(d), v)
+#define fu_device_set_update_error(d, v) fwupd_device_set_update_error(FWUPD_DEVICE(d), v)
+#define fu_device_add_vendor_id(d, v)	 fwupd_device_add_vendor_id(FWUPD_DEVICE(d), v)
+#define fu_device_add_protocol(d, v)	 fwupd_device_add_protocol(FWUPD_DEVICE(d), v)
 #define fu_device_set_version_lowest_raw(d, v)                                                     \
 	fwupd_device_set_version_lowest_raw(FWUPD_DEVICE(d), v)
 #define fu_device_set_version_bootloader_raw(d, v)                                                 \
@@ -1084,6 +1084,27 @@ fu_device_emit_request(FuDevice *self, FwupdRequest *request, FuProgress *progre
 FwupdSecurityAttr *
 fu_device_security_attr_new(FuDevice *self, const gchar *appstream_id) G_GNUC_NON_NULL(1, 2);
 
+gboolean
+fu_device_set_contents(FuDevice *self,
+		       const gchar *filename,
+		       GInputStream *stream,
+		       FuProgress *progress,
+		       GError **error) G_GNUC_NON_NULL(1, 2, 3);
+gboolean
+fu_device_set_contents_bytes(FuDevice *self,
+			     const gchar *filename,
+			     GBytes *blob,
+			     FuProgress *progress,
+			     GError **error) G_GNUC_NON_NULL(1, 2, 3);
+GBytes *
+fu_device_get_contents_bytes(FuDevice *self,
+			     const gchar *filename,
+			     FuProgress *progress,
+			     GError **error) G_GNUC_WARN_UNUSED_RESULT G_GNUC_NON_NULL(1, 2);
+gboolean
+fu_device_query_file_exists(FuDevice *self, const gchar *filename, gboolean *exists, GError **error)
+    G_GNUC_NON_NULL(1, 2, 3);
+
 const gchar *
 fu_device_get_instance_str(FuDevice *self, const gchar *key) G_GNUC_NON_NULL(1, 2);
 void
