@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2021 Richard Hughes <richard@hughsie.com>
+ * Copyright 2021 Richard Hughes <richard@hughsie.com>
  *
- * SPDX-License-Identifier: LGPL-2.1+
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #define G_LOG_DOMAIN "FuProgress"
@@ -80,7 +80,13 @@ enum { SIGNAL_PERCENTAGE_CHANGED, SIGNAL_STATUS_CHANGED, SIGNAL_LAST };
 
 static guint signals[SIGNAL_LAST] = {0};
 
-G_DEFINE_TYPE(FuProgress, fu_progress, G_TYPE_OBJECT)
+static void
+fu_progress_codec_iface_init(FwupdCodecInterface *iface);
+
+G_DEFINE_TYPE_WITH_CODE(FuProgress,
+			fu_progress,
+			G_TYPE_OBJECT,
+			G_IMPLEMENT_INTERFACE(FWUPD_TYPE_CODEC, fu_progress_codec_iface_init))
 
 #define FU_PROGRESS_STEPS_MAX 1000
 
@@ -957,7 +963,12 @@ fu_progress_traceback(FuProgress *self)
 	/* allow override */
 	if (tmp != NULL) {
 		g_autoptr(GError) error_local = NULL;
-		if (!fu_strtoull(tmp, &threshold_ms, 0, G_MAXUINT, &error_local))
+		if (!fu_strtoull(tmp,
+				 &threshold_ms,
+				 0,
+				 G_MAXUINT,
+				 FU_INTEGER_BASE_AUTO,
+				 &error_local))
 			g_warning("invalid threshold value: %s", tmp);
 	}
 
@@ -968,50 +979,34 @@ fu_progress_traceback(FuProgress *self)
 }
 
 static void
-fu_progress_to_string_cb(FuProgress *self, guint idt, GString *str)
+fu_progress_add_string(FwupdCodec *codec, guint idt, GString *str)
 {
+	FuProgress *self = FU_PROGRESS(codec);
+
 	/* not interesting */
 	if (self->id == NULL && self->name == NULL)
 		return;
 
-	if (self->id != NULL)
-		fu_string_append(str, idt, "Id", self->id);
-	if (self->name != NULL)
-		fu_string_append(str, idt, "Name", self->name);
+	fwupd_codec_string_append(str, idt, "Id", self->id);
+	fwupd_codec_string_append(str, idt, "Name", self->name);
 	if (self->percentage != G_MAXUINT)
-		fu_string_append_ku(str, idt, "Percentage", self->percentage);
+		fwupd_codec_string_append_int(str, idt, "Percentage", self->percentage);
 	if (self->status != FWUPD_STATUS_UNKNOWN)
-		fu_string_append(str, idt, "Status", fwupd_status_to_string(self->status));
+		fwupd_codec_string_append(str, idt, "Status", fwupd_status_to_string(self->status));
 	if (self->duration > 0.0001)
-		fu_string_append_ku(str, idt, "DurationMs", self->duration * 1000.f);
-	if (self->step_weighting > 0)
-		fu_string_append_ku(str, idt, "StepWeighting", self->step_weighting);
-	if (self->step_now > 0)
-		fu_string_append_ku(str, idt, "StepNow", self->step_now);
+		fwupd_codec_string_append_int(str, idt, "DurationMs", self->duration * 1000.f);
+	fwupd_codec_string_append_int(str, idt, "StepWeighting", self->step_weighting);
+	fwupd_codec_string_append_int(str, idt, "StepNow", self->step_now);
 	for (guint i = 0; i < self->children->len; i++) {
 		FuProgress *child = g_ptr_array_index(self->children, i);
-		fu_progress_to_string_cb(child, idt + 1, str);
+		fwupd_codec_add_string(FWUPD_CODEC(child), idt + 1, str);
 	}
 }
 
-/**
- * fu_progress_to_string:
- * @self: A #FuProgress
- *
- * Prints the progress for debugging.
- *
- * Return value: (transfer full): string
- *
- * Since: 1.8.7
- **/
-gchar *
-fu_progress_to_string(FuProgress *self)
+static void
+fu_progress_codec_iface_init(FwupdCodecInterface *iface)
 {
-	g_autoptr(GString) str = g_string_new(NULL);
-	fu_progress_to_string_cb(self, 0, str);
-	if (str->len == 0)
-		return NULL;
-	return g_string_free(g_steal_pointer(&str), FALSE);
+	iface->add_string = fu_progress_add_string;
 }
 
 static void
