@@ -22,6 +22,7 @@ class Endian(Enum):
 
 
 class Type(Enum):
+    NONE = None
     U8 = "u8"
     U16 = "u16"
     U24 = "u24"
@@ -160,6 +161,7 @@ class StructObj:
             "ParseInternal": Export.NONE,
             "New": Export.NONE,
             "ToString": Export.NONE,
+            "Default": Export.NONE,
         }
 
     def c_method(self, suffix: str):
@@ -206,7 +208,7 @@ class StructObj:
                 if (
                     item.constant
                     and item.type != Type.STRING
-                    and not (item.type == Type.U8 and item.multiplier)
+                    and not (item.type == Type.U8 and item.n_elements)
                 ):
                     item.add_private_export("Getters")
                 if item.struct_obj:
@@ -230,15 +232,17 @@ class StructObj:
                 if (
                     item.constant
                     and item.type != Type.STRING
-                    and not (item.type == Type.U8 and item.multiplier)
+                    and not (item.type == Type.U8 and item.n_elements)
                 ):
                     item.add_private_export("Getters")
                 if item.struct_obj:
                     item.struct_obj.add_private_export("ValidateInternal")
         elif derive == "New":
             for item in self.items:
-                if item.constant and not (item.type == Type.U8 and item.multiplier):
+                if item.constant and not (item.type == Type.U8 and item.n_elements):
                     item.add_private_export("Setters")
+                if item.struct_obj:
+                    item.struct_obj.add_private_export("New")
 
     def add_public_export(self, derive: str) -> None:
         # Getters and Setters are special as we do not want public exports of const
@@ -270,14 +274,15 @@ class StructItem:
     def __init__(self, obj: StructObj) -> None:
         self.obj: StructObj = obj
         self.element_id: str = ""
-        self.type: Type = Type.U8
+        self.type: Type = Type.NONE
+        self.is_packed: bool = False
         self.enum_obj: Optional[EnumObj] = None
         self.struct_obj: Optional[StructObj] = None
         self.default: Optional[str] = None
         self.constant: Optional[str] = None
         self.padding: Optional[str] = None
         self.endian: Endian = Endian.NATIVE
-        self.multiplier: int = 0
+        self.n_elements: int = 0
         self._bits_size: int = 0
         self._bits_offset: int = 0
         self.offset: int = 0
@@ -315,19 +320,23 @@ class StructItem:
 
     @property
     def size(self) -> int:
-        multiplier = self.multiplier
-        if not multiplier:
-            multiplier = 1
-        if self.type in [Type.U8, Type.I8, Type.STRING, Type.GUID]:
-            return multiplier
+        n_elements = self.n_elements
+        if not n_elements:
+            n_elements = 1
+        if self.struct_obj:
+            return n_elements * self.struct_obj.size
+        if self.type in [Type.U8, Type.I8, Type.STRING]:
+            return n_elements
+        if self.type in [Type.GUID]:
+            return n_elements * 16
         if self.type in [Type.U16, Type.I16]:
-            return multiplier * 2
+            return n_elements * 2
         if self.type == Type.U24:
-            return multiplier * 3
+            return n_elements * 3
         if self.type in [Type.U32, Type.I32]:
-            return multiplier * 4
+            return n_elements * 4
         if self.type in [Type.U64, Type.I64]:
-            return multiplier * 8
+            return n_elements * 8
         return 0
 
     @property
@@ -418,7 +427,7 @@ class StructItem:
             if val.startswith('"') and val.endswith('"'):
                 return val[1:-1]
             raise ValueError(f"string default {val} needs double quotes")
-        if self.type == Type.GUID or (self.type == Type.U8 and self.multiplier):
+        if self.type == Type.GUID or (self.type == Type.U8 and self.n_elements):
             if not val.startswith("0x"):
                 raise ValueError(f"0x prefix for hex number expected, got: {val}")
             if len(val) != (self.size * 2) + 2:
@@ -443,7 +452,7 @@ class StructItem:
     def parse_default(self, val: str) -> None:
         if (
             self.type == Type.U8
-            and self.multiplier
+            and self.n_elements
             and val.startswith("0x")
             and len(val) == 4
         ):
@@ -460,19 +469,17 @@ class StructItem:
     ) -> None:
         # is array
         if val.startswith("[") and val.endswith("]"):
-            typestr, multiplier = val[1:-1].split(";", maxsplit=1)
-            if multiplier.startswith("0x"):
-                self.multiplier = int(multiplier[2:], 16)
+            typestr, n_elements = val[1:-1].split(";", maxsplit=1)
+            if n_elements.startswith("0x"):
+                self.n_elements = int(n_elements[2:], 16)
             else:
-                self.multiplier = int(multiplier)
+                self.n_elements = int(n_elements)
         else:
             typestr = val
 
         # nested struct
         if typestr in struct_objs:
             self.struct_obj = struct_objs[typestr]
-            self.multiplier = self.struct_obj.size
-            self.type = Type.U8
             return
 
         # find the type
@@ -505,15 +512,23 @@ class StructItem:
         # defined types
         try:
             self.type = Type(typestr)
-            if self.type == Type.GUID:
-                self.multiplier = 16
         except ValueError as e:
             raise ValueError(f"invalid type: {typestr}") from e
 
+        # sanity check
+        if (
+            self.enabled
+            and self.is_packed
+            and self.endian == Endian.NATIVE
+            and self.type
+            in [Type.U16, Type.U24, Type.U32, Type.U64, Type.I16, Type.I32, Type.I64]
+        ):
+            raise ValueError(f"endian not specified for packed struct: {typestr}")
+
     def __str__(self) -> str:
         tmp = f"{self.element_id}: "
-        if self.multiplier:
-            tmp += str(self.multiplier)
+        if self.n_elements:
+            tmp += str(self.n_elements)
         tmp += self.type.value
         if self.endian != Endian.NATIVE:
             tmp += self.endian.value
@@ -564,6 +579,7 @@ class Generator:
         repr_type: Optional[str] = None
         derives: List[str] = []
         offset: int = 0
+        struct_seen_b32: bool = False
         bits_offset: int = 0
         struct_cur: Optional[StructObj] = None
         enum_cur: Optional[EnumObj] = None
@@ -630,6 +646,7 @@ class Generator:
                 derives.clear()
                 offset = 0
                 bits_offset = 0
+                struct_seen_b32 = False
                 continue
 
             # check for trailing comma
@@ -660,6 +677,8 @@ class Generator:
                 item._bits_offset = bits_offset
                 item.offset = offset
                 item.element_id = parts[0]
+                if repr_type == "C, packed":
+                    item.is_packed = True
 
                 type_parts = parts[1].split("=", maxsplit=3)
                 try:
@@ -670,17 +689,27 @@ class Generator:
                     )
                 except ValueError as e:
                     raise ValueError(f"{str(e)} on line {line_num}: {line}")
+                if len(type_parts) > 1:
+                    if "Default" not in derives:
+                        raise ValueError(
+                            f"struct requires #[derive(Default)] for line {line_num}: {line}"
+                        )
                 if len(type_parts) == 3:
                     item.parse_constant(type_parts[2])
                 elif len(type_parts) == 2:
                     item.parse_default(type_parts[1])
-                offset += item.size
+                if item.size == 0:
+                    struct_seen_b32 = True
+                if not struct_seen_b32:
+                    offset += item.size
                 bits_offset += item.bits_size
                 struct_cur.items.append(item)
 
         # process the templates here
         subst = {
             "basename": self.basename,
+            "enum_objs": self.enum_objs,
+            "struct_objs": self.struct_objs,
         }
         template_h = self._env.get_template(os.path.basename("fu-rustgen.h.in"))
         template_c = self._env.get_template(os.path.basename("fu-rustgen.c.in"))
