@@ -460,20 +460,13 @@ static gboolean
 fu_ata_device_probe(FuDevice *device, GError **error)
 {
 	FuAtaDevice *self = FU_ATA_DEVICE(device);
+	g_autoptr(FuDevice) scsi_parent = NULL;
 
-	/* check is valid */
-	if (g_strcmp0(fu_udev_device_get_devtype(FU_UDEV_DEVICE(device)), "disk") != 0) {
-		g_set_error(error,
-			    FWUPD_ERROR,
-			    FWUPD_ERROR_NOT_SUPPORTED,
-			    "is not correct devtype=%s, expected disk",
-			    fu_udev_device_get_devtype(FU_UDEV_DEVICE(device)));
-		return FALSE;
-	}
-
-	/* set the physical ID */
-	if (!fu_udev_device_set_physical_id(FU_UDEV_DEVICE(device), "scsi", error))
+	/* set the SCSI physical ID for compat */
+	scsi_parent = fu_device_get_backend_parent_with_subsystem(device, "scsi", error);
+	if (scsi_parent == NULL)
 		return FALSE;
+	fu_device_set_physical_id(device, fu_device_get_backend_id(scsi_parent));
 
 	/* look at the PCI and USB depth to work out if in an external enclosure */
 	self->pci_depth = fu_udev_device_get_subsystem_depth(FU_UDEV_DEVICE(device), "pci");
@@ -494,6 +487,33 @@ fu_ata_device_tf_to_pack_id(struct ata_tf *tf)
 	return (((guint64)lbah) << 24) | (guint64)lba24;
 }
 
+static gboolean
+fu_ata_device_ioctl_buf_cb(FuIoctl *self, gpointer ptr, guint8 *buf, gsize bufsz, GError **error)
+{
+	struct sg_io_hdr *io_hdr = (struct sg_io_hdr *)ptr;
+	io_hdr->dxferp = buf;
+	io_hdr->dxfer_len = bufsz;
+	return TRUE;
+}
+
+static gboolean
+fu_ata_device_ioctl_cdb_cb(FuIoctl *self, gpointer ptr, guint8 *buf, gsize bufsz, GError **error)
+{
+	struct sg_io_hdr *io_hdr = (struct sg_io_hdr *)ptr;
+	io_hdr->cmdp = buf;
+	io_hdr->cmd_len = bufsz;
+	return TRUE;
+}
+
+static gboolean
+fu_ata_device_ioctl_sense_cb(FuIoctl *self, gpointer ptr, guint8 *buf, gsize bufsz, GError **error)
+{
+	struct sg_io_hdr *io_hdr = (struct sg_io_hdr *)ptr;
+	io_hdr->sbp = buf;
+	io_hdr->mx_sb_len = bufsz;
+	return TRUE;
+}
+
 static gboolean
 fu_ata_device_command(FuAtaDevice *self,
 		      struct ata_tf *tf,
@@ -506,6 +526,7 @@ fu_ata_device_command(FuAtaDevice *self,
 	guint8 cdb[SG_ATA_12_LEN] = {0x0};
 	guint8 sb[32] = {0x0};
 	sg_io_hdr_t io_hdr = {0x0};
+	g_autoptr(FuIoctl) ioctl = fu_udev_device_ioctl_new(FU_UDEV_DEVICE(self));
 
 	/* map _TO_DEV to PIO mode */
 	if (dxfer_direction == SG_DXFER_TO_DEV)
@@ -533,29 +554,28 @@ fu_ata_device_command(FuAtaDevice *self,
 	cdb[7] = tf->lbah;
 	cdb[8] = tf->dev;
 	cdb[9] = tf->command;
-	fu_dump_raw(G_LOG_DOMAIN, "CDB", cdb, sizeof(cdb));
-	if (dxfer_direction == SG_DXFER_TO_DEV && dxferp != NULL)
-		fu_dump_raw(G_LOG_DOMAIN, "outgoing_data", dxferp, dxfer_len);
 
 	/* hit hardware */
 	io_hdr.interface_id = 'S';
-	io_hdr.mx_sb_len = sizeof(sb);
 	io_hdr.dxfer_direction = dxfer_direction;
-	io_hdr.dxfer_len = dxfer_len;
-	io_hdr.dxferp = dxferp;
-	io_hdr.cmdp = cdb;
-	io_hdr.cmd_len = SG_ATA_12_LEN;
-	io_hdr.sbp = sb;
 	io_hdr.pack_id = fu_ata_device_tf_to_pack_id(tf);
 	io_hdr.timeout = timeout_ms;
-	if (!fu_udev_device_ioctl(FU_UDEV_DEVICE(self),
-				  SG_IO,
-				  (guint8 *)&io_hdr,
-				  sizeof(io_hdr),
-				  NULL,
-				  FU_ATA_DEVICE_IOCTL_TIMEOUT,
-				  FU_UDEV_DEVICE_IOCTL_FLAG_NONE,
-				  error))
+
+	/* include these when generating the emulation event */
+	fu_ioctl_add_key_as_u16(ioctl, "Request", SG_IO);
+	fu_ioctl_add_key_as_u8(ioctl, "DxferDirection", io_hdr.dxfer_direction);
+	fu_ioctl_add_key_as_u8(ioctl, "PackId", io_hdr.pack_id);
+	fu_ioctl_add_mutable_buffer(ioctl, NULL, dxferp, dxfer_len, fu_ata_device_ioctl_buf_cb);
+	fu_ioctl_add_const_buffer(ioctl, "Cdb", cdb, sizeof(cdb), fu_ata_device_ioctl_cdb_cb);
+	fu_ioctl_add_mutable_buffer(ioctl, "Sense", sb, sizeof(sb), fu_ata_device_ioctl_sense_cb);
+	if (!fu_ioctl_execute(ioctl,
+			      SG_IO,
+			      (guint8 *)&io_hdr,
+			      sizeof(io_hdr),
+			      NULL,
+			      FU_ATA_DEVICE_IOCTL_TIMEOUT,
+			      FU_IOCTL_FLAG_NONE,
+			      error))
 		return FALSE;
 	g_debug("ATA_%u status=0x%x, host_status=0x%x, driver_status=0x%x",
 		io_hdr.cmd_len,
@@ -630,7 +650,7 @@ fu_ata_device_setup(FuDevice *device, GError **error)
 {
 	FuAtaDevice *self = FU_ATA_DEVICE(device);
 	struct ata_tf tf = {0x0};
-	guint8 id[FU_ATA_IDENTIFY_SIZE];
+	guint8 id[FU_ATA_IDENTIFY_SIZE] = {0x0};
 
 	/* get ID block */
 	tf.dev = ATA_USING_LBA;
@@ -800,7 +820,11 @@ fu_ata_device_write_firmware(FuDevice *device,
 
 	/* write each block */
 	fu_progress_set_status(progress, FWUPD_STATUS_DEVICE_WRITE);
-	chunks = fu_chunk_array_new_from_stream(stream, 0x00, chunksz, error);
+	chunks = fu_chunk_array_new_from_stream(stream,
+						FU_CHUNK_ADDR_OFFSET_NONE,
+						FU_CHUNK_PAGESZ_NONE,
+						chunksz,
+						error);
 	if (chunks == NULL)
 		return FALSE;
 	fu_progress_set_id(progress, G_STRLOC);
@@ -894,18 +918,10 @@ fu_ata_device_init(FuAtaDevice *self)
 	fu_udev_device_add_open_flag(FU_UDEV_DEVICE(self), FU_IO_CHANNEL_OPEN_FLAG_READ);
 }
 
-static void
-fu_ata_device_finalize(GObject *object)
-{
-	G_OBJECT_CLASS(fu_ata_device_parent_class)->finalize(object);
-}
-
 static void
 fu_ata_device_class_init(FuAtaDeviceClass *klass)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS(klass);
 	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
-	object_class->finalize = fu_ata_device_finalize;
 	device_class->to_string = fu_ata_device_to_string;
 	device_class->set_quirk_kv = fu_ata_device_set_quirk_kv;
 	device_class->setup = fu_ata_device_setup;
