@@ -481,7 +481,8 @@ fu_dell_dock_ec_get_dock_info(FuDevice *device, GError **error)
 			   map->location == LOCATION_BASE && map->sub_type == 0) {
 			if (oldest_base_pd == 0 ||
 			    device_entry[i].version.version_32 < oldest_base_pd)
-				oldest_base_pd = GUINT32_TO_BE(device_entry[i].version.version_32);
+				oldest_base_pd = GUINT32_TO_BE(/* nocheck:blocked */
+							       device_entry[i].version.version_32);
 			g_debug("\tParsed version: %02x.%02x.%02x.%02x",
 				device_entry[i].version.version_8[0],
 				device_entry[i].version.version_8[1],
@@ -944,6 +945,12 @@ fu_dell_dock_ec_open(FuDevice *device, GError **error)
 {
 	FuDellDockEc *self = FU_DELL_DOCK_EC(device);
 
+	/* sanity check */
+	if (fu_device_get_proxy(device) == NULL) {
+		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED, "no proxy");
+		return FALSE;
+	}
+
 	if (!fu_device_open(fu_device_get_proxy(device), error))
 		return FALSE;
 	if (!self->data->dock_type)
@@ -954,6 +961,11 @@ fu_dell_dock_ec_open(FuDevice *device, GError **error)
 static gboolean
 fu_dell_dock_ec_close(FuDevice *device, GError **error)
 {
+	/* sanity check */
+	if (fu_device_get_proxy(device) == NULL) {
+		g_set_error_literal(error, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED, "no proxy");
+		return FALSE;
+	}
 	return fu_device_close(fu_device_get_proxy(device), error);
 }
 
@@ -988,7 +1000,6 @@ fu_dell_dock_ec_init(FuDellDockEc *self)
 	fu_device_add_protocol(FU_DEVICE(self), "com.dell.dock");
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UPDATABLE);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_SIGNED_PAYLOAD);
-	fu_device_add_private_flag(FU_DEVICE(self), FU_DEVICE_PRIVATE_FLAG_INHIBIT_CHILDREN);
 }
 
 static void
