@@ -16,7 +16,11 @@
 #include "config.h"
 
 #include "fu-dell-dock-common.h"
+#include "fu-dell-dock-ec.h"
+#include "fu-dell-dock-mst.h"
 #include "fu-dell-dock-plugin.h"
+#include "fu-dell-dock-status.h"
+#include "fu-dell-dock-tbt.h"
 
 struct _FuDellDockPlugin {
 	FuPlugin parent_instance;
@@ -231,18 +235,12 @@ fu_dell_dock_plugin_device_registered(FuPlugin *plugin, FuDevice *device)
 static gboolean
 fu_dell_dock_plugin_backend_device_removed(FuPlugin *plugin, FuDevice *device, GError **error)
 {
-	const gchar *device_key = fu_device_get_id(device);
-	FuDevice *dev;
 	FuDevice *parent;
 
-	/* only the device with bridge will be in cache */
-	dev = fu_plugin_cache_lookup(plugin, device_key);
-	if (dev == NULL)
+	if (!FU_IS_USB_DEVICE(device))
 		return TRUE;
-	fu_plugin_cache_remove(plugin, device_key);
 
-	/* find the parent and ask daemon to remove whole chain  */
-	parent = fu_device_get_parent(dev);
+	parent = fu_device_get_parent(device);
 	if (parent != NULL && FU_IS_DELL_DOCK_EC(parent)) {
 		g_debug("Removing %s (%s)", fu_device_get_name(parent), fu_device_get_id(parent));
 		fu_plugin_device_remove(plugin, parent);
@@ -340,6 +338,10 @@ fu_dell_dock_plugin_constructed(GObject *obj)
 	/* allow these to be built by quirks */
 	fu_plugin_add_device_gtype(plugin, FU_TYPE_DELL_DOCK_STATUS);
 	fu_plugin_add_device_gtype(plugin, FU_TYPE_DELL_DOCK_MST);
+	fu_plugin_add_device_gtype(plugin, FU_TYPE_DELL_DOCK_EC);     /* coverage */
+	fu_plugin_add_device_gtype(plugin, FU_TYPE_DELL_DOCK_MST);    /* coverage */
+	fu_plugin_add_device_gtype(plugin, FU_TYPE_DELL_DOCK_STATUS); /* coverage */
+	fu_plugin_add_device_gtype(plugin, FU_TYPE_DELL_DOCK_TBT);    /* coverage */
 
 #ifndef _WIN32
 	/* currently slower performance, but more reliable in corner cases */
