@@ -94,7 +94,7 @@ fu_focalfp_hid_device_io(FuFocalfpHidDevice *self,
 		if (!fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(self),
 						  buf,
 						  sizeof(buf),
-						  FU_UDEV_DEVICE_IOCTL_FLAG_NONE,
+						  FU_IOCTL_FLAG_NONE,
 						  error)) {
 			return FALSE;
 		}
@@ -106,7 +106,7 @@ fu_focalfp_hid_device_io(FuFocalfpHidDevice *self,
 		if (!fu_hidraw_device_get_feature(FU_HIDRAW_DEVICE(self),
 						  buf,
 						  sizeof(buf),
-						  FU_UDEV_DEVICE_IOCTL_FLAG_NONE,
+						  FU_IOCTL_FLAG_NONE,
 						  error)) {
 			return FALSE;
 		}
@@ -488,7 +488,11 @@ fu_focalfp_hid_device_write_firmware(FuDevice *device,
 	fu_progress_step_done(progress);
 
 	/* send packet data */
-	chunks = fu_chunk_array_new_from_stream(stream, 0x0, MAX_USB_PACKET_SIZE, error);
+	chunks = fu_chunk_array_new_from_stream(stream,
+						FU_CHUNK_ADDR_OFFSET_NONE,
+						FU_CHUNK_PAGESZ_NONE,
+						MAX_USB_PACKET_SIZE,
+						error);
 	if (chunks == NULL)
 		return FALSE;
 	if (!fu_focalfp_hid_device_write_chunks(self,
