@@ -40,6 +40,7 @@ struct _FuLogitechBulkcontrollerDevice {
 	gboolean is_sync_transfer_in_progress;
 	GString *device_info_response_json;
 	gsize transfer_bufsz;
+	guint32 sequence_id;
 };
 
 G_DEFINE_TYPE(FuLogitechBulkcontrollerDevice, fu_logitech_bulkcontroller_device, FU_TYPE_USB_DEVICE)
@@ -66,6 +67,7 @@ fu_logitech_bulkcontroller_device_to_string(FuDevice *device, guint idt, GString
 					  "DeviceInfoResponse",
 					  self->device_info_response_json->str);
 	}
+	fwupd_codec_string_append_hex(str, idt, "SequenceId", self->sequence_id);
 }
 
 static gboolean
@@ -227,16 +229,17 @@ static gboolean
 fu_logitech_bulkcontroller_device_sync_send_cmd(FuLogitechBulkcontrollerDevice *self,
 						FuLogitechBulkcontrollerCmd cmd,
 						GByteArray *buf,
-						guint32 *sequence_id,
 						GError **error)
 {
 	g_autoptr(GByteArray) st_req = fu_struct_logitech_bulkcontroller_send_sync_req_new();
 	g_autofree gchar *str = NULL;
-	guint32 sequence_id_tmp = g_random_int_range(0, G_MAXINT32);
+
+	/* increment */
+	self->sequence_id++;
 
 	/* send */
 	fu_struct_logitech_bulkcontroller_send_sync_req_set_cmd(st_req, cmd);
-	fu_struct_logitech_bulkcontroller_send_sync_req_set_sequence_id(st_req, sequence_id_tmp);
+	fu_struct_logitech_bulkcontroller_send_sync_req_set_sequence_id(st_req, self->sequence_id);
 	if (buf != NULL) {
 		fu_struct_logitech_bulkcontroller_send_sync_req_set_payload_length(st_req,
 										   buf->len);
@@ -252,15 +255,12 @@ fu_logitech_bulkcontroller_device_sync_send_cmd(FuLogitechBulkcontrollerDevice *
 		return FALSE;
 
 	/* success */
-	if (sequence_id != NULL)
-		*sequence_id = sequence_id_tmp;
 	return TRUE;
 }
 
 static gboolean
 fu_logitech_bulkcontroller_device_sync_send_ack(FuLogitechBulkcontrollerDevice *self,
 						FuLogitechBulkcontrollerCmd cmd,
-						guint32 *sequence_id,
 						GError **error)
 {
 	g_autoptr(GByteArray) buf_ack = g_byte_array_new();
@@ -268,7 +268,6 @@ fu_logitech_bulkcontroller_device_sync_send_ack(FuLogitechBulkcontrollerDevice *
 	if (!fu_logitech_bulkcontroller_device_sync_send_cmd(self,
 							     FU_LOGITECH_BULKCONTROLLER_CMD_ACK,
 							     buf_ack,
-							     sequence_id,
 							     error)) {
 		g_prefix_error(error,
 			       "failed to send ack for %s: ",
@@ -305,7 +304,9 @@ fu_logitech_bulkcontroller_device_sync_wait_any(FuLogitechBulkcontrollerDevice *
 	g_byte_array_append(response->data,
 			    buf + st->len,
 			    fu_struct_logitech_bulkcontroller_send_sync_res_get_payload_length(st));
-	if (response->data->len == 0) {
+	/* no payload for UninitBuffer, skip check */
+	if ((response->cmd != FU_LOGITECH_BULKCONTROLLER_CMD_UNINIT_BUFFER) &&
+	    (response->data->len == 0)) {
 		g_set_error_literal(error,
 				    FWUPD_ERROR,
 				    FWUPD_ERROR_READ,
@@ -444,7 +445,7 @@ fu_logitech_bulkcontroller_device_sync_wait_ack_cb(FuDevice *device,
 
 	buf = fu_logitech_bulkcontroller_device_sync_wait_cmd(self,
 							      FU_LOGITECH_BULKCONTROLLER_CMD_ACK,
-							      helper->sequence_id,
+							      self->sequence_id,
 							      error);
 	if (buf == NULL)
 		return FALSE;
@@ -459,10 +460,9 @@ fu_logitech_bulkcontroller_device_sync_wait_ack_cb(FuDevice *device,
 static gboolean
 fu_logitech_bulkcontroller_device_sync_wait_ack(FuLogitechBulkcontrollerDevice *self,
 						FuLogitechBulkcontrollerCmd cmd,
-						guint32 sequence_id,
 						GError **error)
 {
-	FuLogitechBulkcontrollerResponse helper = {.cmd = cmd, .sequence_id = sequence_id};
+	FuLogitechBulkcontrollerResponse helper = {.cmd = cmd};
 	return fu_device_retry_full(FU_DEVICE(self),
 				    fu_logitech_bulkcontroller_device_sync_wait_ack_cb,
 				    10,
@@ -472,18 +472,18 @@ fu_logitech_bulkcontroller_device_sync_wait_ack(FuLogitechBulkcontrollerDevice *
 }
 
 static gboolean
-fu_logitech_bulkcontroller_device_sync_check_ack(FuLogitechBulkcontrollerResponse *response,
+fu_logitech_bulkcontroller_device_sync_check_ack(FuLogitechBulkcontrollerDevice *self,
+						 FuLogitechBulkcontrollerResponse *response,
 						 FuLogitechBulkcontrollerCmd cmd,
-						 guint32 sequence_id,
 						 GError **error)
 {
 	/* verify the sequence ID */
-	if (response->sequence_id != sequence_id) {
+	if (response->sequence_id != self->sequence_id) {
 		g_set_error(error,
 			    FWUPD_ERROR,
 			    FWUPD_ERROR_INVALID_DATA,
 			    "sequence ID invalid, expected 0x%04x and got 0x%04x",
-			    sequence_id,
+			    self->sequence_id,
 			    response->sequence_id);
 		return FALSE;
 	}
@@ -495,7 +495,6 @@ fu_logitech_bulkcontroller_device_sync_write(FuLogitechBulkcontrollerDevice *sel
 					     GByteArray *req,
 					     GError **error)
 {
-	guint32 sequence_id = 0;
 	g_autoptr(GByteArray) res_ack = NULL;
 	g_autoptr(GByteArray) res_read = NULL;
 	g_autoptr(GByteArray) buf = NULL;
@@ -505,7 +504,6 @@ fu_logitech_bulkcontroller_device_sync_write(FuLogitechBulkcontrollerDevice *sel
 		self,
 		FU_LOGITECH_BULKCONTROLLER_CMD_BUFFER_WRITE,
 		req,
-		&sequence_id,
 		error)) {
 		g_prefix_error(error, "failed to send request: ");
 		return NULL;
@@ -515,7 +513,6 @@ fu_logitech_bulkcontroller_device_sync_write(FuLogitechBulkcontrollerDevice *sel
 	if (!fu_logitech_bulkcontroller_device_sync_wait_ack(
 		self,
 		FU_LOGITECH_BULKCONTROLLER_CMD_BUFFER_WRITE,
-		sequence_id,
 		error)) {
 		g_prefix_error(error, "failed to wait for ack: ");
 		return NULL;
@@ -526,7 +523,6 @@ fu_logitech_bulkcontroller_device_sync_write(FuLogitechBulkcontrollerDevice *sel
 		self,
 		FU_LOGITECH_BULKCONTROLLER_CMD_UNINIT_BUFFER,
 		NULL,
-		&sequence_id,
 		error)) {
 		g_prefix_error(error, "failed to uninit buffer: ");
 		return NULL;
@@ -549,9 +545,9 @@ fu_logitech_bulkcontroller_device_sync_write(FuLogitechBulkcontrollerDevice *sel
 				return NULL;
 			}
 			if (!fu_logitech_bulkcontroller_device_sync_check_ack(
+				self,
 				response_tmp,
 				FU_LOGITECH_BULKCONTROLLER_CMD_UNINIT_BUFFER,
-				sequence_id,
 				error)) {
 				g_prefix_error(error, "failed to check uninit buffer: ");
 				return NULL;
@@ -573,7 +569,6 @@ fu_logitech_bulkcontroller_device_sync_write(FuLogitechBulkcontrollerDevice *sel
 	if (!fu_logitech_bulkcontroller_device_sync_send_ack(
 		self,
 		FU_LOGITECH_BULKCONTROLLER_CMD_BUFFER_READ,
-		&sequence_id,
 		error)) {
 		g_prefix_error(error, "failed to ack read buffer: ");
 		return NULL;
@@ -594,7 +589,6 @@ fu_logitech_bulkcontroller_device_sync_write(FuLogitechBulkcontrollerDevice *sel
 	if (!fu_logitech_bulkcontroller_device_sync_send_ack(
 		self,
 		FU_LOGITECH_BULKCONTROLLER_CMD_UNINIT_BUFFER,
-		NULL,
 		error)) {
 		g_prefix_error(error, "failed to ack uninit buffer: ");
 		return NULL;
@@ -758,7 +752,6 @@ fu_logitech_bulkcontroller_device_ensure_child(FuLogitechBulkcontrollerDevice *s
 		}
 	}
 
-
 	/* create new child */
 	child = g_object_new(FU_TYPE_LOGITECH_BULKCONTROLLER_CHILD, "proxy", self, NULL);
 	fu_device_add_private_flag(child, FU_DEVICE_PRIVATE_FLAG_REFCOUNTED_PROXY);
@@ -918,7 +911,8 @@ fu_logitech_bulkcontroller_device_ensure_info_cb(FuDevice *device,
 	 */
 	if (send_req) {
 		g_autoptr(GByteArray) device_request =
-		    fu_logitech_bulkcontroller_proto_manager_generate_get_device_info_request();
+		    fu_logitech_bulkcontroller_proto_manager_generate_get_device_info_request(
+			device);
 		buf = fu_logitech_bulkcontroller_device_sync_write(self, device_request, error);
 		if (buf == NULL)
 			return FALSE;
@@ -970,7 +964,8 @@ fu_logitech_bulkcontroller_device_write_fw(FuLogitechBulkcontrollerDevice *self,
 
 	chunks = fu_chunk_array_new_from_stream(
 	    stream,
-	    0x0,
+	    FU_CHUNK_ADDR_OFFSET_NONE,
+	    FU_CHUNK_PAGESZ_NONE,
 	    self->transfer_bufsz - FU_STRUCT_LOGITECH_BULKCONTROLLER_UPDATE_REQ_SIZE,
 	    error);
 	if (chunks == NULL)
@@ -1018,7 +1013,8 @@ fu_logitech_bulkcontroller_device_verify_cb(FuDevice *device, gpointer user_data
 		g_autoptr(GByteArray) device_request = NULL;
 		g_debug("manually requesting as no pending request: %s", error_local->message);
 		device_request =
-		    fu_logitech_bulkcontroller_proto_manager_generate_get_device_info_request();
+		    fu_logitech_bulkcontroller_proto_manager_generate_get_device_info_request(
+			device);
 		buf = fu_logitech_bulkcontroller_device_sync_write(self, device_request, error);
 		if (buf == NULL)
 			return FALSE;
@@ -1192,7 +1188,8 @@ fu_logitech_bulkcontroller_device_set_time_cb(FuDevice *device, gpointer user_da
 
 	/* send SetDeviceTimeRequest to sync device clock with host */
 	device_request =
-	    fu_logitech_bulkcontroller_proto_manager_generate_set_device_time_request(error);
+	    fu_logitech_bulkcontroller_proto_manager_generate_set_device_time_request(device,
+										      error);
 	if (device_request == NULL)
 		return FALSE;
 	buf = fu_logitech_bulkcontroller_device_sync_write(self, device_request, error);
@@ -1244,7 +1241,8 @@ fu_logitech_bulkcontroller_device_transition_to_device_mode_cb(FuDevice *device,
 	g_autoptr(GByteArray) res = NULL;
 	g_autoptr(GByteArray) decoded_pkt = NULL;
 
-	req = fu_logitech_bulkcontroller_proto_manager_generate_transition_to_device_mode_request();
+	req = fu_logitech_bulkcontroller_proto_manager_generate_transition_to_device_mode_request(
+	    device);
 	res = fu_logitech_bulkcontroller_device_sync_write(self, req, error);
 	if (res == NULL)
 		return FALSE;
@@ -1331,7 +1329,6 @@ fu_logitech_bulkcontroller_device_check_buffer_size(FuLogitechBulkcontrollerDevi
 		self,
 		FU_LOGITECH_BULKCONTROLLER_CMD_CHECK_BUFFERSIZE,
 		NULL, /* data */
-		NULL, /* sequence_id */
 		error)) {
 		g_prefix_error(error, "failed to send request: ");
 		return FALSE;
