@@ -135,7 +135,11 @@ fu_logitech_rallysystem_tablehub_device_write_fw(FuLogitechRallysystemTablehubDe
 						 FuProgress *progress,
 						 GError **error)
 {
-	g_autoptr(FuChunkArray) chunks = fu_chunk_array_new_from_stream(stream, 0x0, 0x200, error);
+	g_autoptr(FuChunkArray) chunks = fu_chunk_array_new_from_stream(stream,
+									FU_CHUNK_ADDR_OFFSET_NONE,
+									FU_CHUNK_PAGESZ_NONE,
+									0x200,
+									error);
 	if (chunks == NULL)
 		return FALSE;
 	fu_progress_set_id(progress, G_STRLOC);
@@ -386,7 +390,7 @@ fu_logitech_rallysystem_tablehub_device_init(FuLogitechRallysystemTablehubDevice
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_UPDATABLE);
 	fu_device_add_flag(FU_DEVICE(self), FWUPD_DEVICE_FLAG_SIGNED_PAYLOAD);
 	fu_device_set_install_duration(FU_DEVICE(self), 5 * 60);
-	fu_device_set_remove_delay(FU_DEVICE(self), 60 * 1000); /* wait for subcomponent */
+	fu_device_set_remove_delay(FU_DEVICE(self), 5 * 60 * 1000); /* wait for subcomponent */
 }
 
 static void
