@@ -52,6 +52,7 @@ typedef struct {
 
 struct _FuMediatekScalerDevice {
 	FuDrmDevice parent_instance;
+	guint8 randval_cnt;
 };
 
 G_DEFINE_TYPE(FuMediatekScalerDevice, fu_mediatek_scaler_device, FU_TYPE_DRM_DEVICE)
@@ -198,8 +199,8 @@ fu_mediatek_scaler_device_display_is_connected(FuMediatekScalerDevice *self, GEr
 	g_autoptr(GByteArray) st_res = NULL;
 	g_autoptr(GError) error_local = NULL;
 	guint8 randval_req = 0;
-	guint8 randval1 = g_random_int_range(1, 255);
-	guint8 randval2 = g_random_int_range(1, 255);
+	guint8 randval1 = self->randval_cnt++;
+	guint8 randval2 = self->randval_cnt++;
 
 	fu_struct_ddc_cmd_set_vcp_code(st_req, FU_DDC_VCP_CODE_SUM);
 	fu_byte_array_append_uint8(st_req, randval1);
@@ -489,7 +490,10 @@ fu_mediatek_scaler_device_set_data(FuMediatekScalerDevice *self, FuChunk *chk, G
 	g_autoptr(GBytes) chk_bytes = fu_chunk_get_bytes(chk);
 
 	/* smaller slices to accodomate pch variants */
-	chk_slices = fu_chunk_array_new_from_bytes(chk_bytes, 0x00, DDC_DATA_FRAGEMENT_SIZE);
+	chk_slices = fu_chunk_array_new_from_bytes(chk_bytes,
+						   FU_CHUNK_ADDR_OFFSET_NONE,
+						   FU_CHUNK_PAGESZ_NONE,
+						   DDC_DATA_FRAGEMENT_SIZE);
 	for (guint i = 0; i < fu_chunk_array_length(chk_slices); i++) {
 		g_autoptr(FuChunk) chk_slice = NULL;
 		g_autoptr(GByteArray) st_req = fu_struct_ddc_cmd_new();
@@ -746,7 +750,11 @@ fu_mediatek_scaler_device_write_firmware_impl(FuMediatekScalerDevice *self,
 	guint32 sent_sz = 0x0;
 	g_autoptr(FuChunkArray) chunks = NULL;
 
-	chunks = fu_chunk_array_new_from_stream(stream, 0x00, DDC_DATA_PAGE_SIZE, error);
+	chunks = fu_chunk_array_new_from_stream(stream,
+						FU_CHUNK_ADDR_OFFSET_NONE,
+						FU_CHUNK_PAGESZ_NONE,
+						DDC_DATA_PAGE_SIZE,
+						error);
 	if (chunks == NULL)
 		return FALSE;
 
