@@ -17,11 +17,14 @@ G_DEFINE_TYPE(FuOptionromDevice, fu_optionrom_device, FU_TYPE_PCI_DEVICE)
 static gboolean
 fu_optionrom_device_probe(FuDevice *device, GError **error)
 {
+	gboolean exists_fn = FALSE;
 	g_autofree gchar *fn = NULL;
 
 	/* does the device even have ROM? */
 	fn = g_build_filename(fu_udev_device_get_sysfs_path(FU_UDEV_DEVICE(device)), "rom", NULL);
-	if (!g_file_test(fn, G_FILE_TEST_EXISTS)) {
+	if (!fu_device_query_file_exists(device, fn, &exists_fn, error))
+		return FALSE;
+	if (!exists_fn) {
 		g_set_error(error,
 			    FWUPD_ERROR,
 			    FWUPD_ERROR_NOT_SUPPORTED,
@@ -63,18 +66,10 @@ fu_optionrom_device_init(FuOptionromDevice *self)
 	fu_udev_device_add_open_flag(FU_UDEV_DEVICE(self), FU_IO_CHANNEL_OPEN_FLAG_READ);
 }
 
-static void
-fu_optionrom_device_finalize(GObject *object)
-{
-	G_OBJECT_CLASS(fu_optionrom_device_parent_class)->finalize(object);
-}
-
 static void
 fu_optionrom_device_class_init(FuOptionromDeviceClass *klass)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS(klass);
 	FuDeviceClass *device_class = FU_DEVICE_CLASS(klass);
-	object_class->finalize = fu_optionrom_device_finalize;
 	device_class->dump_firmware = fu_optionrom_device_dump_firmware;
 	device_class->probe = fu_optionrom_device_probe;
 }
