@@ -115,7 +115,7 @@ fu_pxi_wireless_device_get_cmd_response(FuPxiWirelessDevice *device,
 		if (!fu_hidraw_device_get_feature(FU_HIDRAW_DEVICE(parent),
 						  buf,
 						  bufsz,
-						  FU_UDEV_DEVICE_IOCTL_FLAG_NONE,
+						  FU_IOCTL_FLAG_NONE,
 						  error))
 			return FALSE;
 		if (!fu_memread_uint8_safe(buf, bufsz, 0x4, &sn, error))
@@ -181,7 +181,7 @@ fu_pxi_wireless_device_check_crc(FuDevice *device, guint16 checksum, GError **er
 	if (!fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(parent),
 					  receiver_cmd->data,
 					  receiver_cmd->len,
-					  FU_UDEV_DEVICE_IOCTL_FLAG_NONE,
+					  FU_IOCTL_FLAG_NONE,
 					  error))
 		return FALSE;
 	if (!fu_pxi_wireless_device_get_cmd_response(self, buf, sizeof(buf), error))
@@ -256,7 +256,7 @@ fu_pxi_wireless_device_fw_object_create(FuDevice *device, FuChunk *chk, GError *
 	if (!fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(parent),
 					  receiver_cmd->data,
 					  receiver_cmd->len,
-					  FU_UDEV_DEVICE_IOCTL_FLAG_NONE,
+					  FU_IOCTL_FLAG_NONE,
 					  error))
 		return FALSE;
 
@@ -314,7 +314,7 @@ fu_pxi_wireless_device_write_payload(FuDevice *device, FuChunk *chk, GError **er
 	if (!fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(parent),
 					  receiver_cmd->data,
 					  receiver_cmd->len,
-					  FU_UDEV_DEVICE_IOCTL_FLAG_NONE,
+					  FU_IOCTL_FLAG_NONE,
 					  error))
 		return FALSE;
 
@@ -340,6 +340,7 @@ fu_pxi_wireless_device_write_chunk(FuDevice *device, FuChunk *chk, GError **erro
 	/* write payload */
 	chunks = fu_chunk_array_new_from_bytes(chk_bytes,
 					       fu_chunk_get_address(chk),
+					       FU_CHUNK_PAGESZ_NONE,
 					       self->fwstate.mtu_size);
 
 	for (guint i = 0; i < fu_chunk_array_length(chunks); i++) {
@@ -402,7 +403,7 @@ fu_pxi_wireless_device_fw_ota_preceding(FuDevice *device, GError **error)
 	return fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(parent),
 					    receiver_cmd->data,
 					    receiver_cmd->len,
-					    FU_UDEV_DEVICE_IOCTL_FLAG_NONE,
+					    FU_IOCTL_FLAG_NONE,
 					    error);
 }
 
@@ -442,7 +443,7 @@ fu_pxi_wireless_device_fw_ota_init_new(FuDevice *device, gsize bufsz, GError **e
 	if (!fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(parent),
 					  receiver_cmd->data,
 					  receiver_cmd->len,
-					  FU_UDEV_DEVICE_IOCTL_FLAG_NONE,
+					  FU_IOCTL_FLAG_NONE,
 					  error))
 		return FALSE;
 
@@ -501,7 +502,7 @@ fu_pxi_wireless_device_fw_ota_ini_new_check(FuDevice *device, GError **error)
 	if (!fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(parent),
 					  receiver_cmd->data,
 					  receiver_cmd->len,
-					  FU_UDEV_DEVICE_IOCTL_FLAG_NONE,
+					  FU_IOCTL_FLAG_NONE,
 					  error))
 		return FALSE;
 
@@ -598,7 +599,7 @@ fu_pxi_wireless_device_fw_upgrade(FuDevice *device,
 	if (!fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(parent),
 					  receiver_cmd->data,
 					  receiver_cmd->len,
-					  FU_UDEV_DEVICE_IOCTL_FLAG_NONE,
+					  FU_IOCTL_FLAG_NONE,
 					  error))
 		return FALSE;
 
@@ -657,7 +658,7 @@ fu_pxi_wireless_device_reset(FuDevice *device, GError **error)
 	return fu_hidraw_device_set_feature(FU_HIDRAW_DEVICE(parent),
 					    receiver_cmd->data,
 					    receiver_cmd->len,
-					    FU_UDEV_DEVICE_IOCTL_FLAG_NONE,
+					    FU_IOCTL_FLAG_NONE,
 					    error);
 }
 
@@ -694,7 +695,10 @@ fu_pxi_wireless_device_write_firmware(FuDevice *device,
 		return FALSE;
 	fu_progress_step_done(progress);
 
-	chunks = fu_chunk_array_new_from_bytes(fw, 0x0, FU_PXI_DEVICE_OBJECT_SIZE_MAX);
+	chunks = fu_chunk_array_new_from_bytes(fw,
+					       FU_CHUNK_ADDR_OFFSET_NONE,
+					       FU_CHUNK_PAGESZ_NONE,
+					       FU_PXI_DEVICE_OBJECT_SIZE_MAX);
 	/* prepare write fw into device */
 	self->fwstate.offset = 0;
 	self->fwstate.checksum = 0;
