@@ -458,7 +458,8 @@ fu_realtek_mst_device_flash_iface_write(FuRealtekMstDevice *self,
 					GError **error)
 {
 	gsize total_size = g_bytes_get_size(data);
-	g_autoptr(FuChunkArray) chunks = fu_chunk_array_new_from_bytes(data, address, 256);
+	g_autoptr(FuChunkArray) chunks =
+	    fu_chunk_array_new_from_bytes(data, address, FU_CHUNK_PAGESZ_NONE, 256);
 
 	g_debug("write %#" G_GSIZE_MODIFIER "x bytes at %#08x", total_size, address);
 	for (guint i = 0; i < fu_chunk_array_length(chunks); i++) {
@@ -593,11 +594,14 @@ fu_realtek_mst_device_write_firmware(FuDevice *device,
 	guint32 flag_addr = self->active_bank == FU_REALTEK_MST_DEVICE_FLASH_BANK_USER1
 				? FLASH_FLAG2_ADDR
 				: FLASH_FLAG1_ADDR;
-	GBytes *firmware_bytes = fu_firmware_get_bytes(firmware, error);
 	const guint8 flag_data[] = {0xaa, 0xaa, 0xaa, 0xff, 0xff};
 	g_autofree guint8 *readback_buf = g_malloc0(FLASH_USER_SIZE);
+	g_autoptr(GBytes) firmware_bytes = NULL;
 
 	/* sanity check */
+	firmware_bytes = fu_firmware_get_bytes(firmware, error);
+	if (firmware_bytes == NULL)
+		return FALSE;
 	if (g_bytes_get_size(firmware_bytes) != FLASH_USER_SIZE) {
 		g_set_error(error,
 			    FWUPD_ERROR,
@@ -683,6 +687,7 @@ fu_realtek_mst_device_read_firmware(FuDevice *device, FuProgress *progress, GErr
 	FuRealtekMstDevice *self = FU_REALTEK_MST_DEVICE(device);
 	guint32 bank_address;
 	g_autofree guint8 *image_bytes = NULL;
+	g_autoptr(GBytes) firmware_bytes = NULL;
 
 	if (self->active_bank == FU_REALTEK_MST_DEVICE_FLASH_BANK_USER1)
 		bank_address = FLASH_USER1_ADDR;
@@ -707,8 +712,9 @@ fu_realtek_mst_device_read_firmware(FuDevice *device, FuProgress *progress, GErr
 						    progress,
 						    error))
 		return NULL;
-	return fu_firmware_new_from_bytes(
-	    g_bytes_new_take(g_steal_pointer(&image_bytes), FLASH_USER_SIZE));
+	firmware_bytes = g_bytes_new_take(g_steal_pointer(&image_bytes), FLASH_USER_SIZE);
+
+	return fu_firmware_new_from_bytes(firmware_bytes);
 }
 
 static GBytes *
