@@ -564,8 +564,11 @@ fu_synaptics_vmm9_device_write_firmware(FuDevice *device,
 	stream = fu_firmware_get_stream(firmware, error);
 	if (stream == NULL)
 		return FALSE;
-	chunks =
-	    fu_chunk_array_new_from_stream(stream, 0x0, FU_STRUCT_HID_PAYLOAD_SIZE_FIFO, error);
+	chunks = fu_chunk_array_new_from_stream(stream,
+						FU_CHUNK_ADDR_OFFSET_NONE,
+						FU_CHUNK_PAGESZ_NONE,
+						FU_STRUCT_HID_PAYLOAD_SIZE_FIFO,
+						error);
 	if (chunks == NULL)
 		return FALSE;
 	if (!fu_synaptics_vmm9_device_write_blocks(self,
@@ -593,6 +596,13 @@ fu_synaptics_vmm9_device_write_firmware(FuDevice *device,
 	}
 	fu_progress_step_done(progress);
 
+	/* if device reboot is not required */
+	if (fu_device_has_private_flag(device, FU_DEVICE_PRIVATE_FLAG_SKIPS_RESTART)) {
+		fu_device_add_flag(device, FWUPD_DEVICE_FLAG_INSTALL_SKIP_VERSION_CHECK);
+		g_debug("skipped device reboot intentionally.");
+		return TRUE;
+	}
+
 	/* generic request */
 	if (fu_device_has_private_flag(device,
 				       FU_SYNAPTICS_VMM9_DEVICE_FLAG_MANUAL_RESTART_REQUIRED)) {
