@@ -1,7 +1,8 @@
 // Copyright 2024 Richard hughes <Richard@hughsie.com>
 // SPDX-License-Identifier: LGPL-2.1-or-later
 
-#[derive(ValidateStream, ParseStream)]
+#[derive(ValidateStream, ParseStream, Default)]
+#[repr(C, packed)]
 struct FuStructSynapticsVmm9 {
     signature: [char; 7] == "CARRERA",
 }
@@ -46,6 +47,7 @@ enum FuSynapticsVmm9RcSts {
 }
 
 #[derive(New, Getters)]
+#[repr(C, packed)]
 struct FuStructHidPayload {
     cap: u8,
     state: u8,
@@ -56,7 +58,8 @@ struct FuStructHidPayload {
     fifo: [u8; 32],
 }
 
-#[derive(New, ToString, Getters)]
+#[derive(New, ToString, Getters, Default)]
+#[repr(C, packed)]
 struct FuStructHidSetCommand {
     id: u8 == 0x1,
     type: u8 == 0x0, // packet write
@@ -65,7 +68,8 @@ struct FuStructHidSetCommand {
     checksum: u8, // this is actually lower if @rc_fifo is less than 32 bytes
 }
 
-#[derive(New, Parse)]
+#[derive(New, Parse, Default)]
+#[repr(C, packed)]
 struct FuStructHidGetCommand {
     id: u8 == 0x1,
     type: u8 == 0x0, // packet reply
@@ -75,6 +79,7 @@ struct FuStructHidGetCommand {
 }
 
 #[derive(Parse)]
+#[repr(C, packed)]
 struct FuStructSynapticsUpdGetId {
     _pid: u16le,
     cid: u8,
