@@ -31,9 +31,17 @@ fu_uefi_pk_plugin_init(FuUefiPkPlugin *self)
 {
 }
 
+static void
+fu_uefi_pk_plugin_constructed(GObject *obj)
+{
+	FuPlugin *plugin = FU_PLUGIN(obj);
+	fu_plugin_set_device_gtype_default(plugin, FU_TYPE_UEFI_PK_DEVICE); /* coverage */
+}
+
 static void
 fu_uefi_pk_plugin_class_init(FuUefiPkPluginClass *klass)
 {
 	FuPluginClass *plugin_class = FU_PLUGIN_CLASS(klass);
+	plugin_class->constructed = fu_uefi_pk_plugin_constructed;
 	plugin_class->coldplug = fu_uefi_pk_plugin_coldplug;
 }
