@@ -12,13 +12,13 @@
 #include "fu-vli-struct.h"
 
 struct _FuVliPdParadeDevice {
-	FuDevice parent_instance;
+	FuUsbDevice parent_instance;
 	FuVliDeviceKind device_kind;
 	guint8 page2; /* base address */
 	guint8 page7; /* base address */
 };
 
-G_DEFINE_TYPE(FuVliPdParadeDevice, fu_vli_pd_parade_device, FU_TYPE_DEVICE)
+G_DEFINE_TYPE(FuVliPdParadeDevice, fu_vli_pd_parade_device, FU_TYPE_USB_DEVICE)
 
 #define FU_VLI_PD_PARADE_I2C_CMD_WRITE 0xa6
 #define FU_VLI_PD_PARADE_I2C_CMD_READ  0xa5
@@ -501,7 +501,10 @@ fu_vli_pd_parade_device_write_firmware(FuDevice *device,
 		return FALSE;
 	if (!fu_vli_pd_parade_device_wait_ready(self, error))
 		return FALSE;
-	blocks = fu_chunk_array_new_from_bytes(fw, 0x0, 0x10000);
+	blocks = fu_chunk_array_new_from_bytes(fw,
+					       FU_CHUNK_ADDR_OFFSET_NONE,
+					       FU_CHUNK_PAGESZ_NONE,
+					       0x10000);
 	for (guint i = 1; i < fu_chunk_array_length(blocks); i++) {
 		g_autoptr(FuChunk) chk = fu_chunk_array_index(blocks, i, error);
 		if (chk == NULL)
