@@ -542,8 +542,10 @@ fu_wac_device_write_firmware(FuDevice *device,
 			return FALSE;
 
 		/* write block in chunks */
-		chunks =
-		    fu_chunk_array_new_from_bytes(blob_block, fd->start_addr, self->write_block_sz);
+		chunks = fu_chunk_array_new_from_bytes(blob_block,
+						       fd->start_addr,
+						       FU_CHUNK_PAGESZ_NONE,
+						       self->write_block_sz);
 		for (guint j = 0; j < fu_chunk_array_length(chunks); j++) {
 			g_autoptr(FuChunk) chk = NULL;
 			g_autoptr(GBytes) blob_chunk = NULL;
@@ -561,7 +563,8 @@ fu_wac_device_write_firmware(FuDevice *device,
 		}
 
 		/* calculate expected checksum and save to device RAM */
-		csum_local[i] = GUINT32_TO_LE(fu_sum32w_bytes(blob_block, G_LITTLE_ENDIAN));
+		csum_local[i] = GUINT32_TO_LE(/* nocheck:blocked */
+					      fu_sum32w_bytes(blob_block, G_LITTLE_ENDIAN));
 		g_debug("block checksum %02u: 0x%08x", i, csum_local[i]);
 		if (!fu_wac_device_set_checksum_of_block(self, i, csum_local[i], error))
 			return FALSE;
@@ -797,7 +800,8 @@ fu_wac_device_add_modules(FuWacDevice *self, GError **error)
 	/* get versions of each module */
 	number_modules = fu_struct_module_desc_get_number_modules(st_desc);
 	for (guint8 i = 0; i < number_modules; i++) {
-		guint16 ver;
+		guint32 ver;
+		guint16 ver2;
 		FuWacModuleFwType fw_type;
 		g_autofree gchar *name = NULL;
 		g_autoptr(FuStructModuleItem) st_item = NULL;
@@ -806,7 +810,18 @@ fu_wac_device_add_modules(FuWacDevice *self, GError **error)
 		st_item = fu_struct_module_item_parse(buf->data, buf->len, offset, error);
 		if (st_item == NULL)
 			return FALSE;
+
 		ver = fu_struct_module_item_get_version(st_item);
+		ver2 = fu_struct_module_item_get_version2(st_item);
+
+		/*
+		 * When ver2 is available and not 0, it is appended to ver in order to make it BCD
+		 * 32bits, otherwise it stays BCD 16bits.
+		 */
+		if (ver2 != 0xFF && ver2 != 0) {
+			ver = (ver << 16);
+			ver |= (ver2 << 8);
+		}
 
 		fw_type = fu_struct_module_item_get_kind(st_item) & 0x7F;
 		switch (fw_type) {
@@ -943,6 +958,9 @@ fu_wac_device_set_progress(FuDevice *self, FuProgress *progress)
 static gchar *
 fu_wac_device_convert_version(FuDevice *device, guint64 version_raw)
 {
+	if (version_raw > G_MAXUINT16)
+		return fu_version_from_uint32(version_raw, fu_device_get_version_format(device));
+
 	return fu_version_from_uint16(version_raw, fu_device_get_version_format(device));
 }
 
