@@ -1638,7 +1638,6 @@ fu_dbus_daemon_method_emulation_load(FuDbusDaemon *self,
 				     GDBusMethodInvocation *invocation)
 {
 	gint32 fd_handle = 0;
-	g_autoptr(GError) error = NULL;
 	g_autoptr(FuMainAuthHelper) helper = NULL;
 
 	g_variant_get(parameters, "(h)", &fd_handle);
@@ -1695,7 +1694,6 @@ fu_dbus_daemon_method_emulation_save(FuDbusDaemon *self,
 				     GDBusMethodInvocation *invocation)
 {
 	gint32 fd_handle = 0;
-	g_autoptr(GError) error = NULL;
 	g_autoptr(FuMainAuthHelper) helper = NULL;
 
 	g_variant_get(parameters, "(h)", &fd_handle);
@@ -2146,7 +2144,8 @@ fu_dbus_daemon_method_inhibit(FuDbusDaemon *self,
 	/* watch */
 	inhibit = g_new0(FuDbusDaemonSystemInhibit, 1);
 	inhibit->sender = g_strdup(fu_engine_request_get_sender(request));
-	inhibit->id = g_strdup_printf("dbus-%i", g_random_int_range(1, G_MAXINT - 1));
+	inhibit->id =
+	    g_strdup_printf("dbus-%i", g_random_int_range(1, G_MAXINT - 1)); /* nocheck:blocked */
 	inhibit->watcher_id =
 	    g_bus_watch_name_on_connection(self->connection,
 					   fu_engine_request_get_sender(request),
@@ -2196,6 +2195,18 @@ fu_dbus_daemon_method_install(FuDbusDaemon *self,
 		g_debug("got option %s", prop_key);
 		if (g_strcmp0(prop_key, "install-flags") == 0)
 			helper->flags = g_variant_get_uint64(prop_value);
+
+		/* these are all set by libfwupd < 2.0.x; parse for compatibility */
+		if (g_strcmp0(prop_key, "allow-older") == 0 &&
+		    g_variant_get_boolean(prop_value) == TRUE)
+			helper->flags |= FWUPD_INSTALL_FLAG_ALLOW_OLDER;
+		if (g_strcmp0(prop_key, "allow-reinstall") == 0 &&
+		    g_variant_get_boolean(prop_value) == TRUE)
+			helper->flags |= FWUPD_INSTALL_FLAG_ALLOW_REINSTALL;
+		if (g_strcmp0(prop_key, "allow-branch-switch") == 0 &&
+		    g_variant_get_boolean(prop_value) == TRUE)
+			helper->flags |= FWUPD_INSTALL_FLAG_ALLOW_BRANCH_SWITCH;
+
 		g_variant_unref(prop_value);
 	}
 
