@@ -223,6 +223,10 @@ fu_security_attr_get_name(FwupdSecurityAttr *attr)
 		/* TRANSLATORS: Title: Whether BIOS Firmware updates is enabled */
 		return g_strdup(_("BIOS firmware updates"));
 	}
+	if (g_strcmp0(appstream_id, FWUPD_SECURITY_ATTR_ID_AMD_SMM_LOCKED) == 0) {
+		/* TRANSLATORS: Title: Whether firmware is locked down */
+		return g_strdup(_("SMM locked down"));
+	}
 	/* we should not get here */
 	return g_strdup(fwupd_security_attr_get_name(attr));
 }
@@ -404,6 +408,10 @@ fu_security_attr_get_title(FwupdSecurityAttr *attr)
 		/* TRANSLATORS: Title: Whether BIOS Firmware updates is enabled */
 		return _("BIOS Firmware Updates");
 	}
+	if (g_strcmp0(appstream_id, FWUPD_SECURITY_ATTR_ID_AMD_SMM_LOCKED) == 0) {
+		/* TRANSLATORS: Title: Whether firmware is locked down */
+		return g_strdup(_("System Management Mode"));
+	}
 	return NULL;
 }
 
@@ -592,6 +600,10 @@ fu_security_attr_get_description(FwupdSecurityAttr *attr)
 		/* TRANSLATORS: longer description */
 		return _("Enabling firmware updates for the BIOS allows fixing security issues.");
 	}
-
+	if (g_strcmp0(appstream_id, FWUPD_SECURITY_ATTR_ID_AMD_SMM_LOCKED) == 0) {
+		/* TRANSLATORS: longer description */
+		return g_strdup(_("System management mode is used by the firmware to access "
+				  "resident BIOS code and data."));
+	}
 	return NULL;
 }
